#include <stdio.h>

// DKDSP 0.2
// convert a mono, devkit-standard gamecube DSP into stereo,
// Donkey Konga/Star Fox Assault format

// 0.1 - round up to next interleave segment
// 0.2 - correct file length calculation (bytes, not nibbles in DK header)
//     - optionally read filler DK header info from existing file

unsigned long read32(unsigned char * inword) {
    return ((unsigned long)(inword[0])<<24)|
           ((unsigned long)(inword[1])<<16)|
           ((unsigned long)(inword[2])<<8)|
           inword[3];
}

/*
void write32(unsigned long in, unsigned char * out) {
    out[0]=in>>24;
    out[1]=in>>16;
    out[2]=in>>8;
    out[3]=in;
}

unsigned short read16(unsigned char * inword) {
    return ((unsigned short)(inword[0])<<8)|inword[1];
}

void write16(unsigned short in, unsigned char * out) {
    out[0]=in>>8;
    out[1]=in;
}
*/

int main(int argc, char * * argv) {
    unsigned long c;
    FILE * infile, *infile2, * outfile;

    // default header filler (from 29zelda.dsp)
    char inhead[0x60],outhead[0xe0]={
    0x43, 0x73, 0x74, 0x72, 0x00, 0x66, 0x08, 0x00, 0x1F, 0x57, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
    0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0xCC, 0x5E, 0x00, 0x44, 0xAC, 0x40, 0x02, 0x00, 0x00, 0x00, 0x00,

    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,


    0x00, 0x5E, 0xCC, 0x00, 0x00, 0x6D, 0x56, 0xDC, 0x00, 0x00, 0xAC, 0x44, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x02, 0x00, 0x6C, 0x56, 0xDB, 0x00, 0x00, 0x00, 0x02, 0xFE, 0xF2, 0x02, 0xAE,
    0x09, 0x9C, 0xFA, 0xC0, 0x05, 0xA1, 0xFE, 0xE4, 0x0D, 0x44, 0xF9, 0xF5, 0x02, 0x3E, 0x04, 0x56,
    0x0B, 0xC5, 0xFA, 0x88, 0x07, 0x6E, 0xFF, 0xE4, 0x0D, 0xC8, 0xFA, 0x03, 0x00, 0x00, 0x00, 0x61,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3A, 0x00, 0x00, 0xDB, 0x40, 0x00,
    0x64, 0xFF, 0x12, 0x00, 0x00, 0x00, 0x00, 0x00, 0x50, 0xDE, 0x22, 0x00, 0x01, 0x00, 0x00, 0x00
    };
    unsigned char buffer[0x800*16];

    printf("DKDSP 0.2 - GC DSP mono to stereo converter\nby hcs\n\n");

    if (argc != 3 && argc != 4) {printf("usage: %s MONO.DSP STEREO.DSP\n  or\n       %s MONO.DSP STEREO.DSP DKHEADER.DSP\n",argv[0],argv[0]); return 1;}

    if (!(infile=fopen(argv[1],"rb"))) {printf("error opening %s\n",argv[1]); return 1;}
    if (!(outfile=fopen(argv[2],"wb"))) {printf("error opening %s\n",argv[2]); return 1;}

    if (argc == 4) {
        printf("reading filler...");
        if(!(infile2=fopen(argv[3],"rb"))) {printf("error opening %s\n",argv[3]); return 1;}
        fread(outhead,sizeof(outhead),1,infile2);
        fclose(infile2);
        printf("done\n");
    }

    printf("copying header...");

    fread(inhead,sizeof(inhead),1,infile);

    memcpy(outhead+0x20,inhead,sizeof(inhead));

    // double stored number of nibbles (no need, DK header stores BYTES)
    //write32(read32(outhead+0x24)*2,outhead+0x24);

    fwrite(outhead,sizeof(outhead),1,outfile);

    printf("done\ninterleaving mono->stereo...");

    for (c=0; c < (read32(outhead+0x24)/2)/0x800+1; c++) {
        fread(buffer,0x800,1,infile);
        fwrite(buffer,0x800,1,outfile);
        fwrite(buffer,0x800,1,outfile);
    }

    printf("done\n");

    fclose(infile);
    fclose(outfile);

    return 0;
}
